# EZRestrict release 4 by David Bowland
# ./addons/eventscripts/ezrestrict/ezrestrict.py

# >>> To configure this addon please see ezrestrict.cfg <<<

"""
Introduces new server commands to restrict players or teams from weapons and to remove idle weapons. Requires only ES 2.0+
"""


import es
import gamethread
import langlib
import os.path
import playerlib
import random


info = es.AddonInfo()
info.name = 'EZRestrict'
info.version = '4'
info.url = 'http://addons.eventscripts.com/addons/view/ezrestrict'
info.basename = 'ezrestrict'
info.author = 'SuperDave'


dict_team_restrict = {}
dict_player_restrict = {}

tuple_team_names = ('@all', '@t', '@ct')
tuple_team_tags = ('#all', '#t', '#ct')
dict_team_iterators = {'2':'@t', '3':'@ct'}
tuple_weapon_tags = ('#all', '#primary', '#rifle', '#shotgun', '#smg', '#sniper', '#secondary', '#pistol', '#knife', '#grenade')
dict_weapons = {'deagle':{'longname':'Night Hawk .50C', 'tags':['#all', '#secondary', '#pistol']}, 'ak47':{'longname':'CV-47', 'tags':['#all', '#primary', '#rifle']}, 'scout':{'longname':'Schmidt Scout', 'tags':['#all', '#primary', '#rifle', '#sniper']}, 'aug':{'longname':'Bullpup', 'tags':['#all', '#primary', '#rifle']}, 'g3sg1':{'longname':'D3/AU1', 'tags':['#all', '#primary', '#rifle', '#sniper']}, 'galil':{'longname':'IDF Defender', 'tags':['#all', '#primary', '#rifle']}, 'famas':{'longname':'Clarion 5.56', 'tags':['#all', '#primary', '#rifle']}, 'm4a1':{'longname':'Maverick M4A1 Carbine', 'tags':['#all', '#primary', '#rifle']}, 'sg552':{'longname':'Krieg 552', 'tags':['#all', '#primary', '#rifle']}, 'sg550':{'longname':'Krieg 550 Commando', 'tags':['#all', '#primary', '#rifle', '#sniper']}, 'm249':{'longname':'M249', 'tags':['#all', '#primary']}, 'awp':{'longname':'Magnum Sniper Rifle', 'tags':['#all', '#primary', '#rifle', '#sniper']}, 'tmp':{'longname':'Schmidt Machine Pistol', 'tags':['#all', '#primary', '#smg']}, 'mp5navy':{'longname':'KM Sub-Machine Gun', 'tags':['#all', '#primary', '#smg']}, 'glock':{'longname':'9x19mm Sidearm', 'tags':['#all', '#secondary', '#pistol']}, 'elite':{'longname':'.40 Dual Elites', 'tags':['#all', '#secondary', '#pistol']}, 'm3':{'longname':'Leone 12 Gauge Super', 'tags':['#all', '#primary', '#shotgun']}, 'xm1014':{'longname':'Leone YG1265 Auto Shotgun', 'tags':['#all', '#primary', '#shotgun']}, 'usp':{'longname':'KM .45 Tactical', 'tags':['#all', '#secondary', '#pistol']}, 'mac10':{'longname':'Ingram MAC-10', 'tags':['#all', '#primary', '#smg']}, 'ump45':{'longname':'KM UMP45', 'tags':['#all', '#primary', '#smg']}, 'p228':{'longname':'228 Compact', 'tags':['#all', '#secondary', '#pistol']}, 'fiveseven':{'longname':'ES Five-Seven', 'tags':['#all', '#secondary', '#pistol']}, 'p90':{'longname':'ES C90', 'tags':['#all', '#primary', '#smg']}, 'knife':{'longname':'knife', 'tags':['#all', '#knife']}, 'hegrenade':{'longname':'HE grenade', 'tags':['#all', '#grenade']}, 'flashbang':{'longname':'flashbang', 'tags':['#all', '#grenade']}, 'smokegrenade':{'longname':'smoke grenade', 'tags':['#all', '#grenade']}, 'c4':{'longname':'C4', 'tags':[]}}
dict_npc_weapons = {}

dict_events = {'ezrestricted_pickup':{'userid':'short', 'weapon':'string', 'longname':'string', 'weaponindex':'short'}, 'ezrestrict_player':{'userid':'short', 'weapon':'string', 'longname':'string'}, 'ezunrestrict_player':{'userid':'short', 'weapon':'string', 'longname':'string'}}
dict_options = {'ezrestrict_restrict_repickup':[2, 'Number of seconds a weapon will be unavaible for pickup after a restricted player attempts to pick the weapon up'], 'ezrestrict_restrict_removebanned':[1, '0 = no change, 1 = when a player picks up a weapon both team are restricted from the weapon is removed'], 'ezrestrict_announce':[0, '0 = no change, 1 = players receive a console message when they try to pick up a restricted weapon, 2 = players receive a chat area message when they try to pick up a restricted weapon']}
func_lang_text = None


def load():
   """
   Creates server commands
   Checks to make sure critical server variables are created by the config
   """
   global func_lang_text

   if not es.exists('command', 'ezrestrict'):
      es.regcmd('ezrestrict', 'ezrestrict/ezrestrict_cmd', 'ezrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]\nServer command for restricting weapons from players or teams')
   if not es.exists('command', 'ezunrestrict'):
      es.regcmd('ezunrestrict', 'ezrestrict/ezrestrict_cmd', 'ezunrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]\nServer command for unrestricting weapons from players or teams')
   if not es.exists('command', 'ezrestrict_removeidle'):
      es.regcmd('ezrestrict_removeidle', 'ezrestrict/ezrestrict_removeidle_cmd', 'ezrestrict_removeidle [weapon 1] [weapon 2] ... [weapon n]\nServer command for removing idle weapons')
   if not es.exists('command', 'ezrestrict_saferemove'):
      es.regcmd('ezrestrict_saferemove', 'ezrestrict/ezrestrict_saferemove_cmd', 'ezrestrict_saferemove <index>\nServer command for safely removing an entity')

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('ezrestrict') + '/ezrestrict.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/ezrestrict/ezrestrict.cfg')
   else:
      es.dbgmsg(0, 'EZRestrict: Unable to load ezrestrict.cfg! Please ensure it is in the ./ezrestrict/ directory.')

   str_lang_path = es.getAddonPath('ezrestrict') + '/ezrestrict_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'EZRestrict: Unable to load ezrestrict_languages.ini! Please ensure it is in the ./ezrestrict/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang

   load_events()


def es_map_start(event_var):
   """
   Clears dictionaries of team and player restrictions
   Calls load_events
   """
   dict_team_restrict.clear()
   dict_player_restrict.clear()
   dict_npc_weapons.clear()

   load_events()


def item_pickup(event_var):
   """
   Determines if the item picked up is restricted
   Calls execure_restricted on the restricted weapon if necessary
   """
   str_userid = event_var['userid']
   str_team = event_var['es_userteam']
   str_item = event_var['item']
   bool_is_restricted = 0

   if dict_team_restrict.has_key(str_team):
      if str_item in dict_team_restrict[str_team]:
         bool_is_restricted = 1

   if dict_player_restrict.has_key(str_userid):
      if str_item in dict_player_restrict[str_userid]:
         bool_is_restricted = 1

   if bool_is_restricted:
      check_restricted(str_userid, str_item)


def player_team(event_var):
   """Checks the player for restricted weapons when a player changes teams"""
   str_team = event_var['team']
   str_userid = event_var['userid']

   if dict_team_restrict.has_key(str_team):
      for str_weapon in dict_team_restrict[str_team]:
         check_restricted(str_userid, str_weapon)


def player_disconnect(event_var):
   """Removes the disconnecting player from the restriction dictionary"""
   global dict_player_restrict

   str_userid = event_var['userid']
   if dict_player_restrict.has_key(str_userid):
      del dict_player_restrict[str_userid]


def unload():
   """Removes any outstanding restrictions on picking up weapon types"""
   int_userid = es.getuserid()
   if int_userid:
      for str_weapon in dict_npc_weapons:
         es.server.cmd('es_xfire %s %s addoutput \"spawnflags 0 \"' % (int_userid, str_weapon))

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def check_restricted(str_userid, str_weapon):
   """Checks player str_userid for an entity of str_weapon type"""
   int_handle = es.getplayerhandle(str_userid)
   for int_index in es.createentitylist('weapon_' + str_weapon):
      if es.getindexprop(int_index, 'CBaseEntity.m_hOwnerEntity') == int_handle:
         execute_restricted(str_userid, int_index, str_weapon)


def execute_restricted(str_userid, int_index, str_weapon):
   """
   Removes the restricted weapon
   Creates a new weapon that cannot be picked up for a short time
   Switches the player's weapon to the player's best weapon excluding the weapon removed
   Fires the ezrestricted_pickup event
   """
   global dict_npc_weapons

   int_lastgive = 0
   str_weapon_prefix = 'weapon_%s' % str_weapon
   if int(es.ServerVar('ezrestrict_restrict_removebanned')) and dict_team_restrict.has_key('2') and dict_team_restrict.has_key('3'):
      if str_weapon in dict_team_restrict['2'] and str_weapon in dict_team_restrict['3']:
         int_lastgive = -1
   gamethread.delayed(0.2, saferemove, int_index)

   if not int_lastgive:
      es.server.cmd('es_xgive %s %s' % (str_userid, str_weapon_prefix))
      es.server.cmd('es_xfire %s %s addoutput \"spawnflags 2\"' % (str_userid, str_weapon_prefix))
      int_lastgive = int(es.ServerVar('eventscripts_lastgive'))
      if dict_npc_weapons.has_key(str_weapon_prefix):
         dict_npc_weapons[str_weapon_prefix] += 1
      else:
         dict_npc_weapons[str_weapon_prefix] = 1
      gamethread.delayed(es.ServerVar('ezrestrict_restrict_repickup'), remove_owner, str_weapon_prefix)

   if es.createplayerlist(str_userid)[int(str_userid)]['weapon'][7:] == str_weapon:
      es.sexec(str_userid, 'use weapon_c4')
      for str_tag in ('#knife', '#grenade', '#secondary', '#primary'):
         for str_loop_weapon in dict_weapons:
            if str_tag in dict_weapons[str_loop_weapon]['tags'] and str_loop_weapon <> str_weapon:
               es.sexec(str_userid, 'use weapon_%s' % str_loop_weapon)

   es.event('initialize', 'ezrestricted_pickup')
   es.event('setint', 'ezrestricted_pickup', 'userid', int(str_userid))
   es.event('setstring', 'ezrestricted_pickup', 'weapon', str_weapon_prefix)
   es.event('setstring', 'ezrestricted_pickup', 'longname', dict_weapons[str_weapon]['longname'])
   es.event('setint', 'ezrestricted_pickup', 'weaponindex', int_lastgive)
   es.event('fire', 'ezrestricted_pickup')

   if int(es.ServerVar('ezrestrict_announce')):
      str_player_lang = playerlib.getPlayer(str_userid).get('lang')
      dict_tokens = {'weapon':dict_weapons[str_weapon]['longname']}
      if es.ServerVar('ezrestrict_announce') == 2:
         es.tell(str_userid, '#multi', func_lang_text('restricted pick up', dict_tokens, str_player_lang))
      else:
         es.cexec(str_userid, 'echo ' + remove_tags(func_lang_text('restricted pick up', dict_tokens, str_player_lang)))


def remove_owner(str_weapon):
   """Removes the owner on the str_weapon_prefix entities so they can be picked up again"""
   global dict_npc_weapons

   if dict_npc_weapons.has_key(str_weapon):
      dict_npc_weapons[str_weapon] -= 1
      if not dict_npc_weapons[str_weapon]:
         del dict_npc_weapons[str_weapon]
         int_userid = int(es.getuserid())
         if int_userid:
            es.server.cmd('es_xfire %s %s addoutput \"spawnflags 0\"' % (int_userid, str_weapon))


def ezrestrict_saferemove_cmd():
   """
   ezrestrict_saferemove <index>
   Calls saferemove for the index number or displays syntax
   """
   if es.getargc() == 2:
      saferemove(int(es.getargv(1)))
   else:
      es.dbgmsg(0, 'Syntax: ezrestrict_saferemove <index>')


def saferemove(arg_index):
   """Ensures the entity at index arg_index exists before it is removed"""
   if es.createentitylist(arg_index).has_key(arg_index):
      es.server.cmd('es_xremove %s' % arg_index)


def ezrestrict_removeidle_cmd():
   """
   ezrestrict_removeidle [weapon 1] [weapon 2] ... [weapon n]
   Loops through arguments and calls remove_idleweapontype for each weapon
   """
   int_arg_count = es.getargc()
   if int_arg_count > 1:
      list_args = []
      for int_count in range(1, int_arg_count):
         list_args.append(es.getargv(int_count).lower())
   else:
      list_args = ['#all']

   removeidle(list_args)


def removeidle(arg_weapons = '#all'):
   """Calls remove_idleweapontype for each supplied weapon"""
   if isinstance(arg_weapons, str):
      arg_weapons = (arg_weapons,)

   for str_arg in arg_weapons:
      if str_arg in tuple_weapon_tags:
         for str_weapon in dict_weapons:
            if str_arg in dict_weapons[str_weapon]['tags']:
               remove_idleweapontype(str_weapon)
      else:
         if str_arg[0:7] == 'weapon_':
            str_arg = str_arg[7:]
         if dict_weapons.has_key(str_arg):
            remove_idleweapontype(str_arg)
         else:
            es.dbgmsg(0, 'EZRestrict: Unrecognized weapon or weapon type \"%s\"' % str_arg)


def remove_idleweapontype(str_weapon):
   """Removes all idle weapons of the str_weapon type"""
   for int_index in es.createentitylist('weapon_' + str_weapon):
      int_owner = es.getindexprop(int_index, 'CBaseEntity.m_hOwnerEntity')
      if int_owner == -1:
         es.server.cmd('es_xremove %s' % int_index)


def ezrestrict_cmd():
   """
   ezrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]   or   ezunrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]
   Validates the first argument as either a userid or a team
   Calls handle_restriction
   """
   int_arg_count = es.getargc()
   str_command_name = es.getargv(0).lower()
   func_command = {'ezrestrict':do_restrict, 'ezunrestrict':do_unrestrict}[str_command_name]

   if int_arg_count > 2:
      str_player_arg = es.getargv(1).lower()

      list_args = []
      for int_count in range(2, int_arg_count):
         list_args.append(es.getargv(int_count).lower())

      handle_restriction(str_player_arg, list_args, func_command)
   else:
      es.dbgmsg(0, 'Syntax: %s <player/team> <weapon 1> [weapon 2] ... [weapon n]' % str_command_name)


def restrict(arg_players, arg_weapons):
   """Calls handle_restrict with do_restrict"""
   handle_restriction(arg_players, arg_weapons, do_restrict)


def unrestrict(arg_players, arg_weapons):
   """Calls handle_restrict with do_unrestrict"""
   handle_restriction(arg_players, arg_weapons, do_unrestrict)


def handle_restriction(arg_players, arg_weapons, func_command):
   """Calls the restrict or unrestrict function for each weapon and player specified"""
   if isinstance(arg_players, str) or isinstance(arg_players, int):
      arg_players = (arg_players,)
   if isinstance(arg_weapons, str):
      arg_weapons = (arg_weapons,)

   for str_player_arg in arg_players:
      str_valid_arg = ''
      if str_player_arg in tuple_team_tags or str_player_arg in tuple_team_names:
         str_valid_arg = str_player_arg
      else:
         int_userid = int(es.getuserid(str_player_arg))
         if int_userid:
            str_valid_arg = str(int_userid)

      if str_valid_arg:
         for str_weapon_arg in arg_weapons:
            if str_weapon_arg in tuple_weapon_tags:
               for str_weapon in dict_weapons:
                  if str_weapon_arg in dict_weapons[str_weapon]['tags']:
                     func_command(str_valid_arg, str_weapon)
            else:
               if str_weapon_arg[0:7] == 'weapon_':
                  str_weapon_arg = str_weapon_arg[7:]
               if dict_weapons.has_key(str_weapon_arg):
                  func_command(str_valid_arg, str_weapon_arg)
               else:
                  es.dbgmsg(0, 'EZRestrict: Unrecognized weapon or weapon type \"%s\"' % str_weapon_arg)
      else:
         es.dbgmsg(0, 'EZRestrict: Unrecognized player or team \"%s\"' % str_player_arg)


def do_restrict(str_player_arg, str_weapon):
   """
   Applies restriction to team if necessary
   Applies restriction to player or players
   Fires the ezrestrict_player event
   """
   global dict_team_restrict
   global dict_player_restrict

   if str_player_arg in tuple_team_names:
      for str_team in dict_team_iterators:
         if str_player_arg == '@all' or str_player_arg == dict_team_iterators[str_team]:
            if not dict_team_restrict.has_key(str_team):
               dict_team_restrict[str_team] = []
            if str_weapon not in dict_team_restrict[str_team]:
               dict_team_restrict[str_team].append(str_weapon)
               for int_userid in es.getUseridList():
                  if es.getplayerteam(int_userid) == int(str_team):
                     check_restricted(str(int_userid), str_weapon)

   elif str_player_arg in tuple_team_tags:
      for int_userid in es.getUseridList():
         int_team = es.getplayerteam(int_userid)
         if str_player_arg == '#all' or (str_player_arg == '#t' and int_team == 2) or (str_player_arg == '#ct' and int_team == 3):
            do_restrict(str(int_userid), str_weapon)

   else:
      if not dict_player_restrict.has_key(str_player_arg):
         dict_player_restrict[str_player_arg] = []
      if str_weapon not in dict_player_restrict[str_player_arg]:
         dict_player_restrict[str_player_arg].append(str_weapon)

         es.event('initialize', 'ezrestrict_player')
         es.event('setint', 'ezrestrict_player', 'userid', int(str_player_arg))
         es.event('setstring', 'ezrestrict_player', 'weapon', 'weapon_' + str_weapon)
         es.event('setstring', 'ezrestrict_player', 'longname', dict_weapons[str_weapon]['longname'])
         es.event('fire', 'ezrestrict_player')

         check_restricted(str_player_arg, str_weapon)


def do_unrestrict(str_player_arg, str_weapon):
   """
   Removes restriction from team if necessary
   Removes restriction from player or players
   Fires the ezunrestrict_player event
   """
   global dict_team_restrict
   global dict_player_restrict

   if str_player_arg in tuple_team_names:
      for str_team in dict_team_iterators:
         if str_player_arg == '@all' or str_player_arg == dict_team_iterators[str_team]:
            if dict_team_restrict.has_key(str_team):
               if str_weapon in dict_team_restrict[str_team]:
                  dict_team_restrict[str_team].remove(str_weapon)
                  if not dict_team_restrict[str_team]:
                     del dict_team_restrict[str_team]

   elif str_player_arg in tuple_team_tags:
      for int_userid in es.getUseridList():
         int_team = int(es.getplayerteam(int_userid))
         if str_player_arg == '#all' or (str_player_arg == '#t' and int_team == 2) or (str_player_arg == '#ct' and int_team == 3):
            do_unrestrict(str(int_userid), str_weapon)

   else:
      if dict_player_restrict.has_key(str_player_arg):
         if str_weapon in dict_player_restrict[str_player_arg]:
            dict_player_restrict[str_player_arg].remove(str_weapon)
            if not dict_player_restrict[str_player_arg]:
               del dict_player_restrict[str_player_arg]

            es.event('initialize', 'ezunrestrict_player')
            es.event('setint', 'ezunrestrict_player', 'userid', int(str_player_arg))
            es.event('setstring', 'ezunrestrict_player', 'weapon', 'weapon_' + str_weapon)
            es.event('setstring', 'ezunrestrict_player', 'longname', dict_weapons[str_weapon]['longname'])
            es.event('fire', 'ezunrestrict_player')


def load_events():
   """Loads the .res file for the events"""
   if os.path.isfile(es.getAddonPath('ezrestrict') + '/ezrestrict.res'):
      es.loadevents('declare', 'addons/eventscripts/ezrestrict/ezrestrict.res')
   else:
      if not os.path.isfile(es.getAddonPath('ezrestrict') + 'es_ezrestrict_events_db.txt'):
         es.keygroupcreate('ezrestrict_events')
         for str_event in dict_events:
            es.keycreate('ezrestrict_events', str_event)
            for str_event_var in dict_events[str_event]:
               es.keysetvalue('ezrestrict_events', str_event, str_event_var, dict_events[str_event][str_event_var])
         es.keygroupsave('ezrestrict_events', '|ezrestrict')
         es.keygroupdelete('ezrestrict_events')
      es.loadevents('declare', 'addons/eventscripts/ezrestrict/es_ezrestrict_events_db.txt')
      es.dbgmsg(0, 'EZRestrict: Unable to load ezrestrict.res! Please ensure it is in the ./ezrestrict/ directory.')


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'EZRestrict: ezrestrict_languages.ini not found'